import { CommonModule } from '@angular/common';
import { HttpClient, HttpParams, HttpRequest, HttpClientModule } from '@angular/common/http';
import { InjectionToken, Injectable, Inject, NgModule, defineInjectable, inject } from '@angular/core';
import * as _ from 'lodash';
import { forOwn, isArray, mapValues, isBoolean, cloneDeep, omit } from 'lodash';
import { __spread, __extends } from 'tslib';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var EnvironmentConfigService = new InjectionToken('EnvironmentConfigService');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
var HttpMethods = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    PATCH: 'PUT',
    DELETE: 'DELETE',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var methodsWithBody = [HttpMethods.POST, HttpMethods.PUT, HttpMethods.PATCH];
/** @type {?} */
var uploadingSection = ['media'];
var HttpService = /** @class */ (function () {
    function HttpService(http, config) {
        this.http = http;
        this.config = config;
    }
    /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    HttpService.prototype.handle = /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    function (urlParams, method, options) {
        options = this.filterParams(urlParams, options);
        if (methodsWithBody.indexOf(method) === -1) {
            options = {
                params: new HttpParams({
                    fromObject: options
                }),
                responseType: urlParams.responseType || 'json'
            };
        }
        /** @type {?} */
        var request = new HttpRequest(method, this.buildUrl(urlParams), options);
        // add header to prevent token checking for public requests
        if (urlParams.insecure === true) {
            request = request.clone({ setHeaders: { Insecure: '1' } });
            delete urlParams.insecure;
        }
        return this.http.request(request)
            .toPromise()
            .then((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            return {
                result: true,
                data: (urlParams.rawResponse === true) ? event : event.body
            };
        }))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        function (error) {
            return {
                result: false,
                error: (urlParams.rawResponse === true) ? error : error.error
            };
        }));
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.get = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.GET, params);
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.search = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.GET, params);
    };
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    HttpService.prototype.save = /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    function (urlParams, data) {
        return this.handle(urlParams, HttpMethods.POST, data);
    };
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    HttpService.prototype.update = /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    function (urlParams, data) {
        return this.handle(urlParams, HttpMethods.PUT, data);
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.delete = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.DELETE, params);
    };
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @param urlParams
     * @param options
     */
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    HttpService.prototype.filterParams = /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    function (urlParams, options) {
        /** @type {?} */
        var params = {};
        forOwn(options, (/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        function (value, key) {
            if (isArray(value) && key.indexOf('[]') === -1) {
                key += '[]';
            }
            if (value !== null && value !== undefined) {
                params[key] = value;
            }
        }));
        // convert boolean values to numeric except media content
        if (uploadingSection.indexOf(urlParams.section) === -1) {
            params = mapValues(params, (/**
             * @param {?} value
             * @return {?}
             */
            function (value) { return isBoolean(value) ? +value : value; }));
        }
        return params;
    };
    /**
     * Make url like `/section/id/cmd/secondId`
     * @param urlParams
     */
    /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    HttpService.prototype.buildUrl = /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    function (urlParams) {
        /** @type {?} */
        var url = [this.config.url + "/" + urlParams.section];
        if (urlParams.id !== undefined) {
            url.push("" + urlParams.id);
        }
        if (urlParams.cmd) {
            url.push("" + urlParams.cmd);
        }
        if (urlParams.secondId) {
            url.push("" + urlParams.secondId);
        }
        return url.join('/');
    };
    HttpService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    HttpService.ctorParameters = function () { return [
        { type: HttpClient },
        { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] }
    ]; };
    /** @nocollapse */ HttpService.ngInjectableDef = defineInjectable({ factory: function HttpService_Factory() { return new HttpService(inject(HttpClient), inject(EnvironmentConfigService)); }, token: HttpService, providedIn: "root" });
    return HttpService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var RequestTypes = {
    GET: 0,
    SEARCH: 1,
    ADD: 2,
    UPDATE: 3,
    DELETE: 4,
    RESTORE: 5,
};
RequestTypes[RequestTypes.GET] = 'GET';
RequestTypes[RequestTypes.SEARCH] = 'SEARCH';
RequestTypes[RequestTypes.ADD] = 'ADD';
RequestTypes[RequestTypes.UPDATE] = 'UPDATE';
RequestTypes[RequestTypes.DELETE] = 'DELETE';
RequestTypes[RequestTypes.RESTORE] = 'RESTORE';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
var  /**
 * @abstract
 * @template T
 */
GetRequest = /** @class */ (function () {
    function GetRequest(service, queryParams) {
        this.service = service;
        this.queryParams = queryParams;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    GetRequest.prototype.url = /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    function (urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    };
    /**
     * Implementation of the adding `with_trashed` into request
     *
     * @param {boolean} value
     */
    /**
     * Implementation of the adding `with_trashed` into request
     *
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    GetRequest.prototype.withTrashed = /**
     * Implementation of the adding `with_trashed` into request
     *
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    function (value) {
        if (value === void 0) { value = true; }
        (/** @type {?} */ (this)).queryParams.with_trashed = value;
        return (/** @type {?} */ (this));
    };
    return GetRequest;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
ApiGetRequest = /** @class */ (function (_super) {
    __extends(ApiGetRequest, _super);
    function ApiGetRequest(service, queryParams) {
        var _this = _super.call(this, service, queryParams) || this;
        _this.service = service;
        _this.queryParams = queryParams;
        return _this;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} id
     * @return {THIS}
     */
    ApiGetRequest.prototype.id = /**
     * @template THIS
     * @this {THIS}
     * @param {?} id
     * @return {THIS}
     */
    function (id) {
        (/** @type {?} */ (this)).queryParams.id = id;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    ApiGetRequest.prototype.with = /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    function (value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @return {?}
     */
    ApiGetRequest.prototype.execute = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var params = cloneDeep(this.queryParams);
        delete params.id;
        return this.service.get(this.queryParams.id, params || {});
    };
    return ApiGetRequest;
}(GetRequest));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
ApiSearchRequest = /** @class */ (function (_super) {
    __extends(ApiSearchRequest, _super);
    function ApiSearchRequest(service, queryParams) {
        var _this = _super.call(this, service, queryParams) || this;
        _this.service = service;
        _this.queryParams = queryParams;
        return _this;
    }
    // TODO: try to use decorator to change url (as parameter of function) + use declare in service
    // TODO: try to use decorator to change url (as parameter of function) + use declare in service
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.query = 
    // TODO: try to use decorator to change url (as parameter of function) + use declare in service
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    function (value) {
        (/** @type {?} */ (this)).queryParams.query = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.page = /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    function (value) {
        (/** @type {?} */ (this)).queryParams.page = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.perPage = /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    function (value) {
        (/** @type {?} */ (this)).queryParams.per_page = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.filterBy = /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?} value
     * @return {THIS}
     */
    function (field, value) {
        (/** @type {?} */ (this)).queryParams[field] = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?=} isDescending
     * @return {THIS}
     */
    ApiSearchRequest.prototype.orderBy = /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?=} isDescending
     * @return {THIS}
     */
    function (field, isDescending) {
        if (isDescending === void 0) { isDescending = false; }
        (/** @type {?} */ (this)).queryParams.order_by = field;
        (/** @type {?} */ (this)).queryParams.desc = isDescending;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.all = /**
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    function (value) {
        if (value === void 0) { value = true; }
        (/** @type {?} */ (this)).queryParams.all = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    ApiSearchRequest.prototype.with = /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    function (value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    };
    /**
     * @return {?}
     */
    ApiSearchRequest.prototype.execute = /**
     * @return {?}
     */
    function () {
        return this.service.search(this.queryParams, this.urlParams);
    };
    return ApiSearchRequest;
}(GetRequest));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
var  /**
 * @abstract
 * @template T
 */
PostRequest = /** @class */ (function () {
    function PostRequest(service, entity) {
        this.service = service;
        this.entity = entity;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    PostRequest.prototype.url = /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    function (urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    };
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    PostRequest.prototype.data = /**
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    function (data) {
        (/** @type {?} */ (this)).entity = data;
        return (/** @type {?} */ (this));
    };
    // filter(filter: ((data: any) => T) | string[]): ApiBodyRequest<T> {
    // filter(filter: ((data: any) => T) | string[]): ApiBodyRequest<T> {
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} filter
     * @return {THIS}
     */
    PostRequest.prototype.filter = 
    // filter(filter: ((data: any) => T) | string[]): ApiBodyRequest<T> {
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} filter
     * @return {THIS}
     */
    function (filter) {
        if (typeof filter === 'function') {
            (/** @type {?} */ (this)).entity = filter((/** @type {?} */ (this)).entity);
        }
        else {
            (/** @type {?} */ (this)).entity = (/** @type {?} */ (omit((/** @type {?} */ (this)).entity, filter)));
        }
        return (/** @type {?} */ (this));
    };
    return PostRequest;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
ApiAddRequest = /** @class */ (function (_super) {
    __extends(ApiAddRequest, _super);
    function ApiAddRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @return {?}
     */
    ApiAddRequest.prototype.execute = /**
     * @return {?}
     */
    function () {
        return this.service.add(this.entity, this.urlParams);
    };
    return ApiAddRequest;
}(PostRequest));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
ApiUpdateRequest = /** @class */ (function (_super) {
    __extends(ApiUpdateRequest, _super);
    function ApiUpdateRequest() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @return {?}
     */
    ApiUpdateRequest.prototype.execute = /**
     * @return {?}
     */
    function () {
        return this.service.update(this.entity, this.urlParams);
    };
    return ApiUpdateRequest;
}(PostRequest));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var ApiService = /** @class */ (function () {
    function ApiService(httpService) {
        this.httpService = httpService;
        this.section = '';
        this.omittedFields = [];
        this.isInsecure = false;
        this.useAutoConstructor = false;
        this.getSection();
    }
    /**
     * @private
     * @return {?}
     */
    ApiService.prototype.getSection = /**
     * @private
     * @return {?}
     */
    function () {
        this.section = this.section || this.constructor.name
            .toLowerCase()
            .replace('service', '');
    };
    /**
     *
     * @param {number | string} id
     * @param {object} params
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y>>}
     */
    // get<Y>(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<Y>>;
    /**
     *
     * @param {number | string} id
     * @param {object} params
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T>>}
     */
    /**
       *
       * @param {number | string} id
       * @param {object} params
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // get<Y>(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<Y>>;
    /**
     *
     * @param {?} id
     * @param {?=} params
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.get = /**
       *
       * @param {number | string} id
       * @param {object} params
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // get<Y>(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<Y>>;
    /**
     *
     * @param {?} id
     * @param {?=} params
     * @param {?=} urlParams
     * @return {?}
     */
    function (id, params, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.get(url, params);
    };
    /**
     * @param {Y} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y>>}
     */
    // add<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {T} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T>>}
     */
    /**
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // add<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.add = /**
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // add<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    function (data, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService.save(url, this.omitFields(data));
    };
    /**
     *
     * @param {Y} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    // update<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {ApiEntity} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
       *
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<void>>}
       */
    // update<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.update = /**
       *
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<void>>}
       */
    // update<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    function (data, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            id: data.id,
            insecure: this.isInsecure
        };
        /** @type {?} */
        var dataWithoutId = cloneDeep(data);
        delete dataWithoutId.id;
        return this.httpService.update(url, dataWithoutId);
    };
    /**
     *
     * @param {number} id
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.delete = /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    function (id, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.delete(url);
    };
    /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {number[]} ids
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.deleteGroup = /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    function (ids, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .delete(url, {
            'ids[]': ids
        });
    };
    /**
     *
     * @param {number} id
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.restore = /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    function (id, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            id: id,
            cmd: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService.save(url, {});
    };
    /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {number[]} ids
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.restoreGroup = /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    function (ids, urlParams) {
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            id: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService
            .save(url, {
            'ids[]': ids
        });
    };
    /**
     *
     * @param {SearchParams} searchParams
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y[]>>}
     */
    // search<Y extends ApiEntity>(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<Y[]>>>;
    /**
     *
     * @param {SearchParams} searchParams
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T[]>>}
     */
    /**
       *
       * @param {SearchParams} searchParams
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y[]>>}
       */
    // search<Y extends ApiEntity>(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<Y[]>>>;
    /**
     *
     * @param {?} searchParams
     * @param {?=} urlParams
     * @return {?}
     */
    ApiService.prototype.search = /**
       *
       * @param {SearchParams} searchParams
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y[]>>}
       */
    // search<Y extends ApiEntity>(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<Y[]>>>;
    /**
     *
     * @param {?} searchParams
     * @param {?=} urlParams
     * @return {?}
     */
    function (searchParams, urlParams) {
        var _this = this;
        /** @type {?} */
        var url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .search(url, searchParams)
            .then((/**
         * @param {?} response
         * @return {?}
         */
        function (response) {
            if (_this.httpService.config.adaptivePagination && searchParams && searchParams.all) {
                // FIXME: temporary solution until backend will be updated to the new version
                response.data = {
                    current_page: 1,
                    data: response.data,
                    first_page_url: '',
                    from: 1,
                    last_page: 1,
                    last_page_url: '',
                    next_page_url: '',
                    path: '',
                    per_page: response.data.length,
                    prev_page_url: '',
                    to: response.data.length,
                    total: response.data.length
                };
            }
            return response;
        }));
    };
    /**
     * @param {?} method
     * @param {?=} params
     * @return {?}
     */
    ApiService.prototype.request = /**
     * @param {?} method
     * @param {?=} params
     * @return {?}
     */
    function (method, params) {
        if (params === void 0) { params = {}; }
        /** @type {?} */
        var request;
        switch (method) {
            case RequestTypes.SEARCH: {
                request = new ApiSearchRequest(this, params);
                break;
            }
            case RequestTypes.GET: {
                request = new ApiGetRequest(this, params);
                break;
            }
            case RequestTypes.ADD: {
                request = new ApiAddRequest(this, params);
                break;
            }
            case RequestTypes.UPDATE: {
                request = new ApiUpdateRequest(this, params);
                break;
            }
        }
        return request;
    };
    /**
     * @protected
     * @template Y
     * @param {?} data
     * @return {?}
     */
    ApiService.prototype.omitFields = /**
     * @protected
     * @template Y
     * @param {?} data
     * @return {?}
     */
    function (data) {
        return omit.apply(_, __spread([data], this.omittedFields));
    };
    ApiService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    ApiService.ctorParameters = function () { return [
        { type: HttpService }
    ]; };
    /** @nocollapse */ ApiService.ngInjectableDef = defineInjectable({ factory: function ApiService_Factory() { return new ApiService(inject(HttpService)); }, token: ApiService, providedIn: "root" });
    return ApiService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// export * from './lib/services/auth.service';
// export * from './lib/interceptors/api-auth.interceptor';
// export * from './lib/interceptors/api-jwt.interceptor';
var ApiLibModule = /** @class */ (function () {
    function ApiLibModule() {
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ApiLibModule.forRoot = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: ApiLibModule,
            providers: [
                { provide: EnvironmentConfigService, useValue: config },
                HttpService,
                ApiService
                // { provide: HttpService, useClass: 'ApiHttpService' },
                // { provide: AuthService, useClass: 'ApiAuthService' },
            ]
        };
    };
    ApiLibModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [
                        CommonModule,
                        HttpClientModule
                    ],
                    exports: [],
                    providers: [
                        HttpService,
                    ]
                },] }
    ];
    return ApiLibModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiLibModule, HttpMethods, RequestTypes, ApiAddRequest, ApiGetRequest, ApiSearchRequest, ApiUpdateRequest, GetRequest, PostRequest, EnvironmentConfigService, HttpService, ApiService };

//# sourceMappingURL=api-lib.js.map